@@ -33,8 +33,6 @@
  ****************************************************************************/
 
 /*
- * TODO - GetMousePos(POINT * result) from ntconio.c
- * TODO - implement nodelay
  * TODO - improve screen-repainting performance, using implied wraparound to reduce write's
  * TODO - make it optional whether screen is restored or not when non-buffered
  */
@@ -44,59 +42,21 @@
 
 #define CUR TerminalType(my_term).
 
-MODULE_ID("$Id: win_driver.c,v 1.94 2025/08/30 20:50:55 tom Exp $")
+MODULE_ID("$Id: win_driver.c,v 1.113 2025/10/04 16:42:46 tom Exp $")
 
 #define WINMAGIC NCDRV_MAGIC(NCDRV_WINCONSOLE)
 #define EXP_OPTIMIZE 0
 
-extern NCURSES_EXPORT(WORD)   _nc_console_MapColor(bool fore, int color);
-
 #define array_length(a) (sizeof(a)/sizeof(a[0]))
 
-static bool InitConsole(void);
-
 #define AssertTCB() assert(TCB != NULL && (TCB->magic == WINMAGIC))
 #define validateConsoleHandle() (AssertTCB(), console_initialized || \
-                                 InitConsole())
+                                 _nc_mingw_init())
 #define SetSP()     assert(TCB->csp != NULL); sp = TCB->csp; (void) sp
 
-#define GenMap(vKey,key) MAKELONG(key, vKey)
-
-#define AdjustY() (WINCONSOLE.buffered ? 0 : (int) WINCONSOLE.SBI.srWindow.Top)
-/* *INDENT-OFF* */
-static const LONG keylist[] =
-{
-    GenMap(VK_PRIOR,  KEY_PPAGE),
-    GenMap(VK_NEXT,   KEY_NPAGE),
-    GenMap(VK_END,    KEY_END),
-    GenMap(VK_HOME,   KEY_HOME),
-    GenMap(VK_LEFT,   KEY_LEFT),
-    GenMap(VK_UP,     KEY_UP),
-    GenMap(VK_RIGHT,  KEY_RIGHT),
-    GenMap(VK_DOWN,   KEY_DOWN),
-    GenMap(VK_DELETE, KEY_DC),
-    GenMap(VK_INSERT, KEY_IC),
-    GenMap(VK_TAB,    KEY_BTAB)
-};
-static const LONG ansi_keys[] =
-{
-    GenMap(VK_PRIOR,  'I'),
-    GenMap(VK_NEXT,   'Q'),
-    GenMap(VK_END,    'O'),
-    GenMap(VK_HOME,   'H'),
-    GenMap(VK_LEFT,   'K'),
-    GenMap(VK_UP,     'H'),
-    GenMap(VK_RIGHT,  'M'),
-    GenMap(VK_DOWN,   'P'),
-    GenMap(VK_DELETE, 'S'),
-    GenMap(VK_INSERT, 'R'),
-    GenMap(VK_TAB,    'Z')
-};
-/* *INDENT-ON* */
-#define N_INI ((int)array_length(keylist))
-#define FKEYS 24
-#define MAPSIZE (FKEYS + N_INI)
-#define CON_NUMPAIRS 64
+#define AdjustY() (WINCONSOLE.buffered \
+                   ? 0 \
+                   : (int) WINCONSOLE.SBI.srWindow.Top)
 
 /*   A process can only have a single console, so it is safe
      to maintain all the information about it in a single
@@ -373,15 +333,6 @@ find_next_change(SCREEN *sp, int row, int col)
     win->_line[row].firstchar = _NOCHANGE;     \
     win->_line[row].lastchar  = _NOCHANGE
 
-static void
-_nc_console_selectActiveHandle(void)
-{
-    if (WINCONSOLE.lastOut != WINCONSOLE.hdl) {
-	WINCONSOLE.lastOut = WINCONSOLE.hdl;
-	SetConsoleActiveScreenBuffer(WINCONSOLE.lastOut);
-    }
-}
-
 static bool
 restore_original_screen(void)
 {
@@ -565,12 +516,6 @@ wcon_doupdate(TERMINAL_CONTROL_BLOCK * TCB)
     returnCode(result);
 }
 
-#ifdef __MINGW32__
-#define SysISATTY(fd) _isatty(fd)
-#else
-#define SysISATTY(fd) isatty(fd)
-#endif
-
 static bool
 wcon_CanHandle(TERMINAL_CONTROL_BLOCK * TCB,
 	       const char *tname,
@@ -584,9 +529,9 @@ wcon_CanHandle(TERMINAL_CONTROL_BLOCK * TCB,
 
     TCB->magic = WINMAGIC;
 
-    if (tname == NULL || *tname == 0)
+    if (tname == NULL || *tname == 0) {
 	code = TRUE;
-    else if (tname != NULL && *tname == '#') {
+    } else if (tname != NULL && *tname == '#') {
 	/*
 	 * Use "#" (a character which cannot begin a terminal's name) to
 	 * select specific driver from the table.
@@ -602,7 +547,7 @@ wcon_CanHandle(TERMINAL_CONTROL_BLOCK * TCB,
 	}
     } else if (tname != NULL && stricmp(tname, "unknown") == 0) {
 	code = TRUE;
-    } else if (SysISATTY(TCB->term.Filedes)) {
+    } else if (_isatty(TCB->term.Filedes)) {
 	code = TRUE;
     }
 
@@ -851,84 +796,6 @@ wcon_sgmode(TERMINAL_CONTROL_BLOCK * TCB, int setFlag, TTY * buf)
     returnCode(result);
 }
 
-#define MIN_WIDE 80
-#define MIN_HIGH 24
-
-/*
- * In "normal" mode, reset the buffer- and window-sizes back to their original values.
- */
-static void
-_nc_console_set_scrollback(bool normal, CONSOLE_SCREEN_BUFFER_INFO * info)
-{
-    SMALL_RECT rect;
-    COORD coord;
-    bool changed = FALSE;
-
-    T((T_CALLED("win32con::_nc_console_set_scrollback(%s)"),
-       (normal
-	? "normal"
-	: "application")));
-
-    T(("... SBI.srWindow %d,%d .. %d,%d",
-       info->srWindow.Top,
-       info->srWindow.Left,
-       info->srWindow.Bottom,
-       info->srWindow.Right));
-    T(("... SBI.dwSize %dx%d",
-       info->dwSize.Y,
-       info->dwSize.X));
-
-    if (normal) {
-	rect = info->srWindow;
-	coord = info->dwSize;
-	if (memcmp(info, &WINCONSOLE.SBI, sizeof(*info)) != 0) {
-	    changed = TRUE;
-	    WINCONSOLE.SBI = *info;
-	}
-    } else {
-	int high = info->srWindow.Bottom - info->srWindow.Top + 1;
-	int wide = info->srWindow.Right - info->srWindow.Left + 1;
-
-	if (high < MIN_HIGH) {
-	    T(("... height %d < %d", high, MIN_HIGH));
-	    high = MIN_HIGH;
-	    changed = TRUE;
-	}
-	if (wide < MIN_WIDE) {
-	    T(("... width %d < %d", wide, MIN_WIDE));
-	    wide = MIN_WIDE;
-	    changed = TRUE;
-	}
-
-	rect.Left =
-	    rect.Top = 0;
-	rect.Right = (SHORT) (wide - 1);
-	rect.Bottom = (SHORT) (high - 1);
-
-	coord.X = (SHORT) wide;
-	coord.Y = (SHORT) high;
-
-	if (info->dwSize.Y != high ||
-	    info->dwSize.X != wide ||
-	    info->srWindow.Top != 0 ||
-	    info->srWindow.Left != 0) {
-	    changed = TRUE;
-	}
-
-    }
-
-    if (changed) {
-	T(("... coord %d,%d", coord.Y, coord.X));
-	T(("... rect %d,%d - %d,%d",
-	   rect.Top, rect.Left,
-	   rect.Bottom, rect.Right));
-	SetConsoleScreenBufferSize(WINCONSOLE.hdl, coord);	/* dwSize */
-	SetConsoleWindowInfo(WINCONSOLE.hdl, TRUE, &rect);	/* srWindow */
-	_nc_console_get_SBI();
-    }
-    returnVoid;
-}
-
 static int
 wcon_mode(TERMINAL_CONTROL_BLOCK * TCB, int progFlag, int defFlag)
 {
@@ -1003,47 +870,6 @@ wcon_wrap(SCREEN *sp GCC_UNUSED)
 {
 }
 
-static int
-rkeycompare(const void *el1, const void *el2)
-{
-    WORD key1 = (LOWORD((*((const LONG *) el1)))) & 0x7fff;
-    WORD key2 = (LOWORD((*((const LONG *) el2)))) & 0x7fff;
-
-    return ((key1 < key2) ? -1 : ((key1 == key2) ? 0 : 1));
-}
-
-static int
-keycompare(const void *el1, const void *el2)
-{
-    WORD key1 = HIWORD((*((const LONG *) el1)));
-    WORD key2 = HIWORD((*((const LONG *) el2)));
-
-    return ((key1 < key2) ? -1 : ((key1 == key2) ? 0 : 1));
-}
-
-static int
-MapKey(WORD vKey)
-{
-    WORD nKey = 0;
-    void *res;
-    LONG key = GenMap(vKey, 0);
-    int code = -1;
-
-    res = bsearch(&key,
-		  WINCONSOLE.map,
-		  (size_t) (N_INI + FKEYS),
-		  sizeof(keylist[0]),
-		  keycompare);
-    if (res) {
-	key = *((LONG *) res);
-	nKey = LOWORD(key);
-	code = (int) (nKey & 0x7fff);
-	if (nKey & 0x8000)
-	    code = -code;
-    }
-    return code;
-}
-
 static void
 wcon_release(TERMINAL_CONTROL_BLOCK * TCB)
 {
@@ -1056,90 +882,6 @@ wcon_release(TERMINAL_CONTROL_BLOCK * TCB)
     returnVoid;
 }
 
-static bool
-read_screen_data(void)
-{
-    bool result = FALSE;
-    COORD bufferCoord;
-    size_t want;
-
-    WINCONSOLE.save_size.X = (SHORT) (WINCONSOLE.save_region.Right
-				      - WINCONSOLE.save_region.Left + 1);
-    WINCONSOLE.save_size.Y = (SHORT) (WINCONSOLE.save_region.Bottom
-				      - WINCONSOLE.save_region.Top + 1);
-
-    want = (size_t) (WINCONSOLE.save_size.X * WINCONSOLE.save_size.Y);
-
-    if ((WINCONSOLE.save_screen = malloc(want * sizeof(CHAR_INFO))) != NULL) {
-	bufferCoord.X = (SHORT) (WINCONSOLE.window_only
-				 ? WINCONSOLE.SBI.srWindow.Left
-				 : 0);
-	bufferCoord.Y = (SHORT) (WINCONSOLE.window_only
-				 ? WINCONSOLE.SBI.srWindow.Top
-				 : 0);
-
-	T(("... reading console %s %dx%d into %d,%d - %d,%d at %d,%d",
-	   WINCONSOLE.window_only ? "window" : "buffer",
-	   WINCONSOLE.save_size.Y, WINCONSOLE.save_size.X,
-	   WINCONSOLE.save_region.Top,
-	   WINCONSOLE.save_region.Left,
-	   WINCONSOLE.save_region.Bottom,
-	   WINCONSOLE.save_region.Right,
-	   bufferCoord.Y,
-	   bufferCoord.X));
-
-	if (read_screen(WINCONSOLE.hdl,
-			WINCONSOLE.save_screen,
-			WINCONSOLE.save_size,
-			bufferCoord,
-			&WINCONSOLE.save_region)) {
-	    result = TRUE;
-	} else {
-	    T((" error %#lx", (unsigned long) GetLastError()));
-	    FreeAndNull(WINCONSOLE.save_screen);
-	}
-    }
-
-    return result;
-}
-
-/*
- * Attempt to save the screen contents.  PDCurses does this if
- * PDC_RESTORE_SCREEN is set, giving the same visual appearance on
- * restoration as if the library had allocated a console buffer.  MSDN
- * says that the data which can be read is limited to 64Kb (and may be
- * less).
- */
-static bool
-save_original_screen(void)
-{
-    bool result = FALSE;
-
-    WINCONSOLE.save_region.Top = 0;
-    WINCONSOLE.save_region.Left = 0;
-    WINCONSOLE.save_region.Bottom = (SHORT) (WINCONSOLE.SBI.dwSize.Y - 1);
-    WINCONSOLE.save_region.Right = (SHORT) (WINCONSOLE.SBI.dwSize.X - 1);
-
-    if (read_screen_data()) {
-	result = TRUE;
-    } else {
-
-	WINCONSOLE.save_region.Top = WINCONSOLE.SBI.srWindow.Top;
-	WINCONSOLE.save_region.Left = WINCONSOLE.SBI.srWindow.Left;
-	WINCONSOLE.save_region.Bottom = WINCONSOLE.SBI.srWindow.Bottom;
-	WINCONSOLE.save_region.Right = WINCONSOLE.SBI.srWindow.Right;
-
-	WINCONSOLE.window_only = TRUE;
-
-	if (read_screen_data()) {
-	    result = TRUE;
-	}
-    }
-
-    T(("... save original screen contents %s", result ? "ok" : "err"));
-    return result;
-}
-
 static void
 wcon_init(TERMINAL_CONTROL_BLOCK * TCB)
 {
@@ -1147,11 +889,11 @@ wcon_init(TERMINAL_CONTROL_BLOCK * TCB)
 
     AssertTCB();
 
-    if (TCB) {
-	if (!InitConsole()) {
-	    returnVoid;
-	}
+    if (!(console_initialized = _nc_mingw_init())) {
+	returnVoid;
+    }
 
+    if (TCB) {
 	TCB->info.initcolor = TRUE;
 	TCB->info.canchange = FALSE;
 	TCB->info.hascolor = TRUE;
@@ -1365,188 +1107,6 @@ wcon_initacs(TERMINAL_CONTROL_BLOCK * TCB,
     }
 }
 
-static ULONGLONG
-tdiff(FILETIME fstart, FILETIME fend)
-{
-    ULARGE_INTEGER ustart;
-    ULARGE_INTEGER uend;
-    ULONGLONG diff;
-
-    ustart.LowPart = fstart.dwLowDateTime;
-    ustart.HighPart = fstart.dwHighDateTime;
-    uend.LowPart = fend.dwLowDateTime;
-    uend.HighPart = fend.dwHighDateTime;
-
-    diff = (uend.QuadPart - ustart.QuadPart) / 10000;
-    return diff;
-}
-
-static int
-Adjust(int milliseconds, int diff)
-{
-    if (milliseconds != NC_INFINITY) {
-	milliseconds -= diff;
-	if (milliseconds < 0)
-	    milliseconds = 0;
-    }
-    return milliseconds;
-}
-
-#define BUTTON_MASK (FROM_LEFT_1ST_BUTTON_PRESSED | \
-                     FROM_LEFT_2ND_BUTTON_PRESSED | \
-                     FROM_LEFT_3RD_BUTTON_PRESSED | \
-                     FROM_LEFT_4TH_BUTTON_PRESSED | \
-                     RIGHTMOST_BUTTON_PRESSED)
-
-static mmask_t
-decode_mouse(SCREEN *sp, int mask)
-{
-    mmask_t result = 0;
-
-    (void) sp;
-    assert(sp && console_initialized);
-
-    if (mask & FROM_LEFT_1ST_BUTTON_PRESSED)
-	result |= BUTTON1_PRESSED;
-    if (mask & FROM_LEFT_2ND_BUTTON_PRESSED)
-	result |= BUTTON2_PRESSED;
-    if (mask & FROM_LEFT_3RD_BUTTON_PRESSED)
-	result |= BUTTON3_PRESSED;
-    if (mask & FROM_LEFT_4TH_BUTTON_PRESSED)
-	result |= BUTTON4_PRESSED;
-
-    if (mask & RIGHTMOST_BUTTON_PRESSED) {
-	switch (WINCONSOLE.numButtons) {
-	case 1:
-	    result |= BUTTON1_PRESSED;
-	    break;
-	case 2:
-	    result |= BUTTON2_PRESSED;
-	    break;
-	case 3:
-	    result |= BUTTON3_PRESSED;
-	    break;
-	case 4:
-	    result |= BUTTON4_PRESSED;
-	    break;
-	}
-    }
-
-    return result;
-}
-
-static int
-_nc_console_twait(
-		     SCREEN *sp,
-		     HANDLE fd,
-		     int mode,
-		     int milliseconds,
-		     int *timeleft
-		     EVENTLIST_2nd(_nc_eventlist * evl))
-{
-    INPUT_RECORD inp_rec;
-    BOOL b;
-    DWORD nRead = 0, rc = (DWORD) (-1);
-    int code = 0;
-    FILETIME fstart;
-    FILETIME fend;
-    int diff;
-    bool isImmed = (milliseconds == 0);
-
-#ifdef NCURSES_WGETCH_EVENTS
-    (void) evl;			/* TODO: implement wgetch-events */
-#endif
-
-#define CONSUME() read_keycode(fd,&inp_rec,1,&nRead)
-
-    assert(sp);
-
-    TR(TRACE_IEVENT, ("start twait: %d milliseconds, mode: %d",
-		      milliseconds, mode));
-
-    if (milliseconds < 0)
-	milliseconds = NC_INFINITY;
-
-    memset(&inp_rec, 0, sizeof(inp_rec));
-
-    while (true) {
-	GetSystemTimeAsFileTime(&fstart);
-	rc = WaitForSingleObject(fd, (DWORD) milliseconds);
-	GetSystemTimeAsFileTime(&fend);
-	diff = (int) tdiff(fstart, fend);
-	milliseconds = Adjust(milliseconds, diff);
-
-	if (!isImmed && milliseconds <= 0)
-	    break;
-
-	if (rc == WAIT_OBJECT_0) {
-	    if (mode) {
-		b = GetNumberOfConsoleInputEvents(fd, &nRead);
-		if (b && nRead > 0) {
-		    b = PeekConsoleInput(fd, &inp_rec, 1, &nRead);
-		    if (b && nRead > 0) {
-			switch (inp_rec.EventType) {
-			case KEY_EVENT:
-			    if (mode & TW_INPUT) {
-				WORD vk = inp_rec.Event.KeyEvent.wVirtualKeyCode;
-				WORD ch = inp_rec.Event.KeyEventChar;
-
-				if (inp_rec.Event.KeyEvent.bKeyDown) {
-				    if (0 == ch) {
-					int nKey = MapKey(vk);
-					if (nKey < 0) {
-					    CONSUME();
-					    continue;
-					}
-				    }
-				    code = TW_INPUT;
-				    goto end;
-				} else {
-				    CONSUME();
-				}
-			    }
-			    continue;
-			case MOUSE_EVENT:
-			    if (decode_mouse(sp,
-					     (inp_rec.Event.MouseEvent.dwButtonState
-					      & BUTTON_MASK)) == 0) {
-				CONSUME();
-			    } else if (mode & TW_MOUSE) {
-				code = TW_MOUSE;
-				goto end;
-			    }
-			    continue;
-			    /* e.g., FOCUS_EVENT */
-			default:
-			    CONSUME();
-			    _nc_console_selectActiveHandle();
-			    continue;
-			}
-		    }
-		}
-	    }
-	    continue;
-	} else {
-	    if (rc != WAIT_TIMEOUT) {
-		code = -1;
-		break;
-	    } else {
-		code = 0;
-		break;
-	    }
-	}
-    }
-  end:
-
-    TR(TRACE_IEVENT, ("end twait: returned %d (%d), remaining time %d msec",
-		      code, errno, milliseconds));
-
-    if (timeleft)
-	*timeleft = milliseconds;
-
-    return code;
-}
-
 static int
 wcon_twait(TERMINAL_CONTROL_BLOCK * TCB,
 	   int mode,
@@ -1620,24 +1180,11 @@ wcon_cursorSet(TERMINAL_CONTROL_BLOCK * TCB GCC_UNUSED, int mode)
 static bool
 wcon_kyExist(TERMINAL_CONTROL_BLOCK * TCB GCC_UNUSED, int keycode)
 {
-    WORD nKey;
-    void *res;
     bool found = FALSE;
-    LONG key = GenMap(0, (WORD) keycode);
 
     T((T_CALLED("win32con::wcon_kyExist(%d)"), keycode));
-    res = bsearch(&key,
-		  WINCONSOLE.rmap,
-		  (size_t) (N_INI + FKEYS),
-		  sizeof(keylist[0]),
-		  rkeycompare);
-    if (res) {
-	key = *((LONG *) res);
-	nKey = LOWORD(key);
-	if (!(nKey & 0x8000))
-	    found = TRUE;
-    }
-    returnCode(found);
+    found = _nc_console_keyExist(keycode);
+    returnBool(found);
 }
 
 static int
@@ -1665,30 +1212,13 @@ wcon_keyok(TERMINAL_CONTROL_BLOCK * TCB,
 {
     int code = ERR;
     SCREEN *sp;
-    WORD nKey;
-    WORD vKey;
-    void *res;
-    LONG key = GenMap(0, (WORD) keycode);
 
     T((T_CALLED("win32con::wcon_keyok(%p, %d, %d)"), TCB, keycode, flag));
 
     if (validateConsoleHandle()) {
 	SetSP();
-
 	if (sp) {
-	    res = bsearch(&key,
-			  WINCONSOLE.rmap,
-			  (size_t) (N_INI + FKEYS),
-			  sizeof(keylist[0]),
-			  rkeycompare);
-	    if (res) {
-		key = *((LONG *) res);
-		vKey = HIWORD(key);
-		nKey = (LOWORD(key)) & 0x7fff;
-		if (!flag)
-		    nKey |= 0x8000;
-		*(LONG *) res = GenMap(vKey, nKey);
-	    }
+	    code = _nc_console_keyok(keycode, flag);
 	}
     }
     returnCode(code);
@@ -1734,71 +1264,10 @@ NCURSES_EXPORT_VAR (TERM_DRIVER) _nc_WIN_DRIVER = {
 	wcon_cursorSet		/* cursorSet     */
 };
 
+#ifdef _NCURSES_MINGW_H
+
 /* --------------------------------------------------------- */
 
-static HANDLE
-get_handle(int fd)
-{
-    intptr_t value = _get_osfhandle(fd);
-    return (HANDLE) value;
-}
-
-/*   Borrowed from ansicon project.
-     Check whether or not an I/O handle is associated with
-     a Windows console.
-*/
-static BOOL
-IsConsoleHandle(HANDLE hdl)
-{
-    DWORD dwFlag = 0;
-    BOOL result;
-
-    if (!GetConsoleMode(hdl, &dwFlag)) {
-	result = (int) WriteConsoleA(hdl, NULL, 0, &dwFlag, NULL);
-    } else {
-	result = (int) (dwFlag & ENABLE_PROCESSED_OUTPUT);
-    }
-    return result;
-}
-
-/*   Our replacement for the systems _isatty to include also
-     a test for mintty. This is called from the NC_ISATTY macro
-     defined in curses.priv.h
- */
-NCURSES_EXPORT(int)
-_nc_mingw_isatty(int fd)
-{
-    int result = 0;
-
-    if (SysISATTY(fd)) {
-	result = 1;
-    } else {
-#if WINVER >= 0x0600
-	result = _nc_console_checkmintty(fd, NULL);
-#endif
-    }
-    return result;
-}
-
-/*   This is used when running in terminfo mode to discover,
-     whether or not the "terminal" is actually a Windows
-     Console. It is the responsibility of the console to deal
-     with the terminal escape sequences that are sent by
-     terminfo.
- */
-NCURSES_EXPORT(int)
-_nc_console_test(int fd)
-{
-    HANDLE hdl = get_handle(fd);
-    int code = 0;
-
-    T((T_CALLED("win32con::_nc_console_test(%d)"), fd));
-
-    code = (int) IsConsoleHandle(hdl);
-
-    returnCode(code);
-}
-
 #define TC_PROLOGUE(fd) \
     SCREEN *sp;                                               \
     TERMINAL *term = NULL;                                    \
@@ -1813,6 +1282,7 @@ _nc_console_test(int fd)
     }                                                         \
     assert(term != NULL)
 
+/* replace by _nc_console_setmode */
 NCURSES_EXPORT(int)
 _nc_mingw_tcsetattr(
 		       int fd,
@@ -1823,7 +1293,7 @@ _nc_mingw_tcsetattr(
 
     if (_nc_console_test(fd)) {
 	DWORD dwFlag = 0;
-	HANDLE ofd = get_handle(fd);
+	HANDLE ofd = _nc_console_handle(fd);
 	if (ofd != INVALID_HANDLE_VALUE) {
 	    if (arg) {
 		if (arg->c_lflag & ICANON)
@@ -1852,6 +1322,7 @@ _nc_mingw_tcsetattr(
     return code;
 }
 
+/* replace by _nc_console_getmode */
 NCURSES_EXPORT(int)
 _nc_mingw_tcgetattr(int fd, struct termios *arg)
 {
@@ -1863,141 +1334,4 @@ _nc_mingw_tcgetattr(int fd, struct termios *arg)
     }
     return code;
 }
-
-NCURSES_EXPORT(int)
-_nc_mingw_tcflush(int fd, int queue)
-{
-    int code = ERR;
-
-    if (_nc_console_test(fd)) {
-	if (queue == TCIFLUSH) {
-	    code = (FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE))
-		    ? OK
-		    : (int) GetLastError());
-	}
-    }
-    return code;
-}
-
-NCURSES_EXPORT(int)
-_nc_mingw_testmouse(
-		       SCREEN *sp,
-		       HANDLE fd,
-		       int delay
-		       EVENTLIST_2nd(_nc_eventlist * evl))
-{
-    int rc = 0;
-
-    assert(sp);
-
-    if (sp->_drv_mouse_head < sp->_drv_mouse_tail) {
-	rc = TW_MOUSE;
-    } else {
-	rc = _nc_console_twait(sp,
-			       fd,
-			       TWAIT_MASK,
-			       delay,
-			       (int *) 0
-			       EVENTLIST_2nd(evl));
-    }
-    return rc;
-}
-
-static bool
-InitConsole(void)
-{
-    {
-	/* initialize once, or not at all */
-	if (!console_initialized) {
-	    int i;
-	    DWORD num_buttons;
-	    WORD a;
-	    BOOL buffered = TRUE;
-	    BOOL b;
-
-	    START_TRACE();
-
-	    WINCONSOLE.map = (LPDWORD) malloc(sizeof(DWORD) * MAPSIZE);
-	    WINCONSOLE.rmap = (LPDWORD) malloc(sizeof(DWORD) * MAPSIZE);
-	    WINCONSOLE.ansi_map = (LPDWORD) malloc(sizeof(DWORD) * MAPSIZE);
-
-	    for (i = 0; i < (N_INI + FKEYS); i++) {
-		if (i < N_INI) {
-		    WINCONSOLE.rmap[i] = WINCONSOLE.map[i] =
-			(DWORD) keylist[i];
-		    WINCONSOLE.ansi_map[i] = (DWORD) ansi_keys[i];
-		} else {
-		    WINCONSOLE.rmap[i] = WINCONSOLE.map[i] =
-			(DWORD) GenMap((VK_F1 + (i - N_INI)),
-				       (KEY_F(1) + (i - N_INI)));
-		    WINCONSOLE.ansi_map[i] =
-			(DWORD) GenMap((VK_F1 + (i - N_INI)),
-				       (';' + (i - N_INI)));
-		}
-	    }
-	    qsort(WINCONSOLE.ansi_map,
-		  (size_t) (MAPSIZE),
-		  sizeof(keylist[0]),
-		  keycompare);
-	    qsort(WINCONSOLE.map,
-		  (size_t) (MAPSIZE),
-		  sizeof(keylist[0]),
-		  keycompare);
-	    qsort(WINCONSOLE.rmap,
-		  (size_t) (MAPSIZE),
-		  sizeof(keylist[0]),
-		  rkeycompare);
-
-	    if (GetNumberOfConsoleMouseButtons(&num_buttons)) {
-		WINCONSOLE.numButtons = (int) num_buttons;
-	    } else {
-		WINCONSOLE.numButtons = 1;
-	    }
-
-	    a = _nc_console_MapColor(true, COLOR_WHITE) |
-		_nc_console_MapColor(false, COLOR_BLACK);
-	    for (i = 0; i < CON_NUMPAIRS; i++)
-		WINCONSOLE.pairs[i] = a;
-
-	    {
-		b = AllocConsole();
-
-		if (!b)
-		    b = AttachConsole(ATTACH_PARENT_PROCESS);
-
-		WINCONSOLE.inp = GetDirectHandle("CONIN$", FILE_SHARE_READ);
-		WINCONSOLE.out = GetDirectHandle("CONOUT$", FILE_SHARE_WRITE);
-
-		if (getenv("NCGDB") || getenv("NCURSES_CONSOLE2")) {
-		    T(("... will not buffer console"));
-		    buffered = FALSE;
-		    WINCONSOLE.hdl = WINCONSOLE.out;
-		} else {
-		    T(("... creating console buffer"));
-		    WINCONSOLE.hdl = CreateConsoleScreenBuffer(GENERIC_READ | GENERIC_WRITE,
-							       FILE_SHARE_READ | FILE_SHARE_WRITE,
-							       NULL,
-							       CONSOLE_TEXTMODE_BUFFER,
-							       NULL);
-		}
-
-		if (WINCONSOLE.hdl != INVALID_HANDLE_VALUE) {
-		    WINCONSOLE.buffered = buffered;
-		    _nc_console_get_SBI();
-		    WINCONSOLE.save_SBI = WINCONSOLE.SBI;
-		    if (!buffered) {
-			save_original_screen();
-			_nc_console_set_scrollback(FALSE, &WINCONSOLE.SBI);
-		    }
-		    GetConsoleCursorInfo(WINCONSOLE.hdl, &WINCONSOLE.save_CI);
-		    T(("... initial cursor is %svisible, %d%%",
-		       (WINCONSOLE.save_CI.bVisible ? "" : "not-"),
-		       (int) WINCONSOLE.save_CI.dwSize));
-		}
-	    }
-
-	    console_initialized = TRUE;
-	}
-    }
-    return (WINCONSOLE.hdl != INVALID_HANDLE_VALUE);
-}
+#endif
